/***************************************************************************
 *            main.c
 *
 *  Fri Oct 24 11:27:05 2003
 *  Copyright  2003  Phil Kett
 *  pkett@genesis-midi.com
 ****************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <dirent.h>
#include <string.h>
#include "../include/remsync_crc.h"
#include "../include/remsync.h"


struct config conf;

int
main (int argc, char *argv[])
{
	int copy_all = 0;
	int result, pid, pidfile;
	int cmd_server = 0, cmd_remote = 0;
	int cmd_default_port = 0, default_port = DEFAULT_PORT;
	int def_conf_found = 0;
	int client_socket = 0;
	char ch;
	char cmd_src_dir[BIG_BUF];
	char cmd_dest_dir[BIG_BUF];
	char cmd_hostname[BIG_BUF];
	char cmd_ip_list[BIG_BUF];
	char cmd_notify[BIG_BUF];
	char pidmsg[BIG_BUF];
#ifdef HAVE_FNOTIFY
	int cmd_monitor = 0;
#endif
	char log_msg[BIG_BUF];
	char *optstr = "Ss:d:p:hH:c:i:n:m";
	FILE *client_file;



	/* Zero all variables for later checking */
	bzero (cmd_src_dir, sizeof (cmd_src_dir));
	bzero (cmd_dest_dir, sizeof (cmd_dest_dir));
	bzero (cmd_hostname, sizeof (cmd_hostname));
	bzero (cmd_ip_list, sizeof (cmd_ip_list));
	bzero (cmd_notify, BIG_BUF);
	bzero (conf.notify, BIG_BUF);
	bzero (conf.source, BIG_BUF);
	bzero (conf.dest, BIG_BUF);
	bzero (conf.ip_list, BIG_BUF);
	bzero (conf.host, BIG_BUF);
	bzero (conf.config_file, BIG_BUF);
	bzero (conf.pidfile, BIG_BUF);
	conf.restart = 0;
	conf.port = default_port;
	conf.server = conf.remote = 0;
#ifdef HAVE_FNOTIFY
	conf.monitor = 0;
#endif


	strcpy (conf.pidfile, DEFAULT_PIDFILE);
	openlog ("Remsync", LOG_PID, LOG_DAEMON);
	/* Check if we've got the default config file */
	if (file_exists (DEFAULT_CONF))
	{
		def_conf_found = 1;
		scan_config_file (DEFAULT_CONF);
	}
	else
	{
		logmsg ("No default config found\n");
		//printf ("No default config file found - continuing\n");
	}



	/* check Arguments */
	if (argc == 1 && def_conf_found == 0)
	{
		/*
		 * No arguments on command line 
		 * and no config file found, starting with default
		 * behaviour (Starting server on default port)
		 */
		start_server (default_port);
		exit (0);
	}
	while (-1 != (ch = getopt (argc, argv, optstr)))
	{
		switch (ch)
		{
		case 'p':
			//printf ("Port Num %s\n", optarg);
			cmd_default_port = atoi (optarg);
			/* Handle the case where only -p is specified
			 * This implies running a server on the specified port
			 */
			if (argc == 3)
			{
				/* There are only 3 args - as we already know that -p has been
				 * specified we can assume that no other options have been used
				 */
				cmd_server = 1;
			}
			break;
		case 'S':
			//printf ("Start Server\n");
			cmd_server = 1;
			break;
		case 's':
			//printf ("Source dir %s\n", optarg);
			strcpy (cmd_src_dir, optarg);
			checkslash (cmd_src_dir);
			break;
		case 'd':
			//printf ("Dest dir %s\n", optarg);
			strcpy (cmd_dest_dir, optarg);
			checkslash (cmd_dest_dir);
			break;
		case 'n':
			strcpy (cmd_notify, optarg);
			break;
		case 'H':
			//printf ("Hostname %s\n", optarg);
			strcpy (cmd_hostname, optarg);
			cmd_remote = 1;
			break;
		case 'm':
#ifdef HAVE_FNOTIFY
		cmd_monitor = 1;
#else 
			printf ("File system monitoring is not supported on this platform\n");
			exit (-1);
#endif
			break;
		case 'h':
			usage ();
			break;
		case 'c':
			if (file_exists (optarg))
			{
				strcpy (conf.config_file, optarg);
				scan_config_file (optarg);
			}
			else
			{
				sprintf (log_msg,
					 "Configuration file %s doesn't exist - exiting...\n",
					 optarg);
				logmsg (log_msg);
				exit (-1);
			}
			break;
		case 'i':
			strncpy (cmd_ip_list, optarg, BIG_BUF);
			if (strlen (optarg) > BIG_BUF)
			{
				cmd_ip_list[BIG_BUF] = '\0';
			}
			break;
		case '?':
			switch (optopt)
			{
			case 'p':
				printf ("Option -p requires port as argument\n");
				break;
			case 'H':
				printf ("Option -h requires hostname as argument\n");
				break;
			case 'd':
				printf ("Option -d requires destination dir as argument\n");
				break;
			case 's':
				printf ("Option -s requires source directory as argument\n");
				break;
			default:
				printf ("Unrecognised option: %c\n", optopt);
				break;
			}
			usage ();
			exit (-1);
		}
	}

	/*
	 * We've now read in the default config (if it exists) and a custom config
	 * if it's specified on the command line. We've also got all the command line parameters 
	 * stored in the cmd_ variables. To complete configuration we now need to copy the cmd_ 
	 * variables into the conf structure.
	 */

	if (strlen (cmd_src_dir) != 0)
	{
		strncpy (conf.source, cmd_src_dir, BIG_BUF);
	}
	if (strlen (cmd_dest_dir) != 0)
	{
		strncpy (conf.dest, cmd_dest_dir, BIG_BUF);
	}
	if (strlen (cmd_hostname) != 0)
	{
		strncpy (conf.host, cmd_hostname, BIG_BUF);
	}
	if (strlen (cmd_ip_list) != 0)
	{
		strncpy (conf.ip_list, cmd_ip_list, BIG_BUF);
	}
	if (cmd_default_port != 0)
	{
		conf.port = cmd_default_port;
	}
	if (strlen (cmd_notify) != 0)
	{
		strncpy (conf.notify, cmd_notify, BIG_BUF);
	}
#ifdef HAVE_FNOTIFY
	conf.monitor = cmd_monitor;
#endif
	if (cmd_server == 1)
	{
		bzero (conf.host, BIG_BUF);
		conf.server = 1;
	}
//      dump_config ();
	/* 
	 * Having dealt with all that we now need to do some checks to make sure that 
	 * we have all the information we need and that nothing is wrong with the config
	 * that we've ended up with!
	 */

	/* First off if conf.hostname is set we need to set conf.remote */
	if (strlen (conf.host) != 0)
	{
		conf.remote = 1;
	}

	/* Sanity check - can't run client and server at the same time */
	if (conf.remote == 1 && conf.server == 1)
	{
		/* 
		 * Specifying remote host and running a server doesn't make sense!
		 * Exit with error message
		 */
		printf ("Incompatible options selected\n");
		usage ();
	}

#ifdef HAVE_FNOTIFY
	/* if we are doing filesystem monitoring we need to make sure
	 * that we have all the stuff we need!
	 */
	if (conf.monitor == 1)
	{
		if (strlen (conf.source) == 0 || strlen (conf.dest) == 0)
		{
			printf ("Source and destination directories needed for monitoring\n");
			exit (-1);
		}
		/*
		 *      we are running a monitor on the source directory
		 * so we kick that off here.
		 * We'll fork the process so that we can detach it from the terminal and run it in the 
		 * background
		 */

		/* Fork and start the set_monitor function */
		/* daemonize! */
		if (daemon(0,0) < 0)
		{
			perror("daemon");
			exit(2);
		}	
	
		if (strlen(conf.pidfile) != 0) {
			/* write out pidfile */
			pidfile = open (conf.pidfile, O_CREAT | O_TRUNC | O_WRONLY | O_BINARY, 0777);
			pid = getpid();
			sprintf(pidmsg, "%d", pid);
			write(pidfile, pidmsg, strlen(pidmsg));
			close(pidfile);	
		}
		
		set_monitor(conf.source);
		exit(0);
	}

#endif


	if (conf.server == 1)
	{
		/*
		 * We're running a server - no other processing is needed 
		 */
		start_server (conf.port);
		exit (0);
	}
	/*
	 * Start processing directories, first check which directories exist.
	 */

	if (conf.remote == 1)
	{
		/* Running as client */

		/* First check we have all the necessary
		 * parameters
		 */
		if (strlen (conf.source) == 0)
		{
			/* no source directory specified */
			printf ("Source Directory not specified");
			exit (1);
		}
		else if (strlen (conf.dest) == 0)
		{
			/* No dest directory specified assuming defaults */
			printf ("Destination directory not specified, assuming it's the same as Source Dir\n%s\n", conf.source);
			strcpy (conf.dest, conf.source);
		}

	}
	if (conf.remote != 1)
	{
		/* 
		 * we're not running a client so the src dir 
		 * should be local
		 */
		if (file_exists (conf.source))
		{
			if (!is_a_dir (conf.source))
			{
				printf ("%s isn't a directory\n",
					conf.source);
				exit (-1);
			}
			if (strcmp (conf.source, conf.dest) == 0)
			{
				printf ("Running locally and source directory is same as destination\n");
				exit (-1);
			}
		}
		else
		{
			printf ("Source Directory %s doesn't exist.\n Exiting....\n", conf.source);
			exit (-1);
		}
	}
	if (file_exists (conf.dest))
	{
		if (!is_a_dir (conf.dest))
		{
			/*
			 * The destination isn't a directory
			 */
			printf ("%s isn't a directory\n", conf.dest);
			exit (-1);
		}
	}
	else
	{
		/*
		 * The destination doesn't already exist.
		 * This means that we can create the directory and copy everything
		 * without any crc checking
		 */
		copy_all = 1;
		printf ("Destination Directory doesn't exist.\nCreating ...");
		result = mkdir (conf.dest, (S_IRWXU | S_IRWXG | S_IRWXO));
		if (result != 0)
		{
			printf ("Failed to create directory %s\n", conf.dest);
			exit (-1);
		}
	}
	if (conf.remote == 1)
	{
		/* run client stuff */
		client_socket = init_client (conf.port, conf.host);
		/* Run rest of client side stuff */
		client_file = fdopen (client_socket, "rw");
		/* 
		 * client_dir_scan needs to be rewritten to deal with the conf structure itself
		 * rather than passing parameters
		 */
		client_dir_scan (client_socket, conf.source,
				 conf.dest, client_file,
				 conf.port, conf.host);
		client_quit (client_socket);
		fclose (client_file);
		close (client_socket);
	}
	else
	{
		/* run local directory scan */
		dir_scan (conf.source, conf.dest, copy_all);
	}
	return 1;
}

void
strip_string (char *buf)
{
	int len, loop;
	char x;
	len = strlen (buf);
	//printf ("String is %d chars long\n", len);
	for (loop = 0; loop <= len; loop++)
	{
		x = buf[loop];
		if (x == '\r' || x == '\n')
		{
			buf[loop] = '\0';
		}
	}
}

void
split_string (char *buf, char *dest1, char *dest2)
{
	int loop = 0, split = 0, len;
	len = strlen (buf);
	for (loop = 0; loop <= len; loop++)
	{
		if ((buf[loop] == ':' && split == 0))
		{
			split = loop + 1;
			dest1[loop] = '\0';
		}
		if (split == 0)
		{
			dest1[loop] = buf[loop];
		}
		else
		{
			dest2[loop - split] = buf[loop];
		}
	}
}


void
client_quit (int client_socket)
{
	char message[BIG_BUF];
	//printf ("Sending Quit\n");
	sprintf (message, "quit\r\n");
	send (client_socket, message, strlen (message), 0);
	//close (client_socket);
}

void
strip_space (char *buffer)
{
	char buffer1[BIG_BUF];
	int len, count, count1;
	strcpy (buffer1, buffer);
	len = strlen (buffer1);
	count1 = 0;
	for (count = 0; count <= len; count++)
	{
		if (buffer1[count] != ' ')
		{
			buffer[count1] = buffer1[count];
			count1++;
		}
	}
}


void
sigchld_handler (int s)
{
	while (wait (NULL) > 0) ;
}

void
sighup_handler (int s)
{
	//printf ("Got SIGHUP\n");
	scan_config_file (DEFAULT_CONF);
	if (strlen (conf.config_file) != 0)
	{
		scan_config_file (conf.config_file);
	}
}
int
daemon (int nochdir, int noclose)
{
	switch (fork ())
	{
	case 0:
		break;
	case -1:
		return -1;
	default:
		_exit (0);	/* exit the original process */
	}

	if (setsid () < 0)	/* shoudn't fail */
		return -1;
	/* dyke out this switch if you want to acquire a control tty in */
	/* the future -- not normally advisable for daemons */
	switch (fork ())
	{
	case 0:
		break;
	case -1:
		return -1;
	default:
		_exit (0);
	}

	if (!nochdir)
		chdir ("/");
	if (!noclose)
	{
		closeall (0);
		open ("/dev/null", O_RDWR);
		dup (0);
		dup (0);
	}

	return 0;
}

void
closeall (int fd)
{
	int fdlimit = sysconf (_SC_OPEN_MAX);
	while (fd < fdlimit)
		close (fd++);
}

void
logmsg (char *message)
{
	syslog (LOG_INFO, "%s failed: %d (%m)", message, errno);
}

void conv_src_to_dest( char *src, char *dest, char *fullfile, char *fulldest) {
	/* Convert a full path source to a full path dest */
	char file[BIG_BUF];
    int len_src, loop1, loop2, lenfull;
    
    len_src = strlen(src);
    lenfull = strlen(fullfile);
    loop2 = 0;
    for (loop1 = len_src; loop1 < lenfull; loop1++) {
        file[loop2] = fullfile[loop1];
        loop2++;
    }
    file[loop2] = '\0';
    
    sprintf(fulldest, "%s%s", dest, file);
}
