/***************************************************************************
 *            process_recv.c
 *
 *  Fri Oct 24 11:28:00 2003
 *  Copyright  2003  Phil Kett
 *  pkett@genesis-midi.com
 ****************************************************************************/

#include "../include/remsync.h"
#include <sys/socket.h>

int
process_recv (char *buf, int fd)
{
	int len, bytes, pid;
	int inF, result, size, checksum, loop, mode, isdir;
	char command[BIG_BUF], argument[BIG_BUF], path[BIG_BUF];
	char message[BIG_BUF], amode[BIG_BUF], fulldest[BIG_BUF], *buffer;
	struct stat filestat;
	bzero (message, BIG_BUF);
	bzero (command, BIG_BUF);
	bzero (argument, BIG_BUF);
	/* first of all we need to remove the trailing \r\n */
	strip_string (buf);
	len = strlen (buf);
	syslog(LOG_INFO, "Got %s\n", buf);
	/* Now we can process the string we've received */
	split_string (buf, command, argument);
	/* 
	 * we now have a command and an argument in seperate strings 
	 * now to actually make them do something!
	 * rewrite this bit to make it a switch statement
	 */
	pid = getpid();
	//printf("%d: Command is '%s' '%s'\n",pid, command, argument);
	if (strcmp (command, "quit") == 0)
	{
		/*
		 * We received a quit request
		 */
		//printf("%d: Got Quit - returning 2\n",pid);
		return 2;
	}
	else if (strcmp (command, "csum") == 0)
	{
		int checksum = 0;
		/* checksum a file
		 * first we need to make sure that it's an ordinary file
		 */
		if (file_exists (argument) && !is_a_dir (argument))
		{
			/*
			 * We now know that the file exists and isn't a directory
			 */
			checksum = cksum (argument);
			//printf ("%d: Checksum of %s is %d\n",pid,  argument, checksum);
			/* Construct reply and send over network */
			sprintf (message, "CSUM:%s:%d\r\n", argument,
				 checksum);
			send (fd, message, strlen (message), 0);
		}
		else
		{
			sprintf (message, "Error: Cannot checksum %s\r\n",
				 argument);
			send (fd, message, strlen (message), 0);
		}
	}
	else if (strcmp (command, "scan") == 0)
	{
		/*
		 * Scan a directory structure and return the checksums
		 */
		if (is_a_dir (argument))
		{
			syslog(LOG_INFO, "Initiating scan of %s\n", argument);
			net_dir_scan (fd, argument, 0);
			sprintf(message, "DONE\r\n");
			send (fd, message, strlen(message), 0);
		}
		else
		{
			sprintf (message,
				 "Error: Cannot scan %s: Not a directory\r\n",
				 argument);
			send (fd, message, strlen (message), 0);
		}
	}
	else if (strcmp (command, "send") == 0)
	{
		/*
		 * Send file over the network
		 */

		/*
		 * first check the argument we've been given is valid
		 */
		if (is_a_dir (argument))
		{
			sprintf (message,
				 "Error: Cannot send %s: Not an ordinary file\r\n",
				 argument);
			send (fd, message, strlen (message), 0);
		}
		else
		{
			/* 
			 * Now we send the file over the network
			 */
			/* 
			 * do a stat to get file sizes and a checksum we can send
			 */
			result = stat (argument, &filestat);
			if (result != 0)
			{
				sprintf (message,
					 "Error: Cannot stat file %s\r\n",
					 argument);
				send (fd, message, strlen (message), 0);
				return 2;
			}
			size = filestat.st_size;
			checksum = cksum (argument);
			bzero (message, BIG_BUF);
			syslog(LOG_INFO, "%d: Sending file %s\n", pid, argument);
			sprintf (message, "Sending:%s:%d:%d\r\n", argument,
				 size, checksum);
			send (fd, message, strlen (message), 0);

			do
			{
				bzero (message, BIG_BUF);
				read (fd, message, BIG_BUF);
				strip_string (message);
			}
			while (strcmp (message, "OK") != 0);

			inF = open (argument, O_RDONLY | O_BINARY);
			buffer = malloc(size);
			bzero(buffer,size);
			bytes = read(inF, buffer, size);
			if (bytes != size) {
				logmsg("file read error");
			}
			bytes = 	write(fd, buffer, size);
			free(buffer);
			
			close (inF);
		}
	}
	else if (strcmp (command, "recv") == 0)
	{
		syslog(LOG_INFO, "%d: Got recv command %s\n", pid, argument);
		/* Receive file from client */
		result = server_recv_file(argument, fd);
	} 
	else if (strcmp(command, "mdir") == 0) {
		/* Make a directory */
		/* first of all split out the mode and the path */
		logmsg("Got mdir command");
		split_string(argument, path, amode);
		mode = atoi(amode);
		syslog(LOG_INFO, "About to run mkdir(%s, %d)\n", path, mode);
		result = mkdir(path, mode);
		if (result != 0) {
			logmsg("Mkdir failed\n");
		}
		bzero(message, BIG_BUF);
		sprintf (message, "OK\r\n");
		send (fd, message, strlen (message), 0);
	}
	else if (strcmp(command, "dele") == 0) {
		/* Delete a file */
		/* first we need to check if this is a directory */
		isdir = is_a_dir(argument);
		if (isdir == 1) {
			/* We have a directory to deal with */
			rmdir(argument);    /* that probably won't work very well.... */
			bzero(message, BIG_BUF);
			sprintf (message, "OK\r\n");
			send (fd, message, strlen (message), 0);
		} else {
			/* It's a file */
			result = unlink(argument);
			if (result != 0) {
				logmsg("Unlink failed");
			}
			bzero(message, BIG_BUF);
			sprintf (message, "OK\r\n");
			send (fd, message, strlen (message), 0);
		}
	}
	else if (strcmp (command, "stat") == 0)
	{

	}
	//printf ("Command '%s'\n", command);
	//printf ("Argument '%s'\n", argument);

	return 1;
}
